# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Devanagari} (and fuzzy permutations)
# 
# Meaning: Script 'DEVANAGARI'
#
return <<'END';
0901	0939	DEVANAGARI
093C	094D	DEVANAGARI
0950	0954	DEVANAGARI
0958	0963	DEVANAGARI
0966	096F	DEVANAGARI
END
