# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{JoinControl} (and fuzzy permutations)
# 
# Meaning: Extended property 'Join_Control'
#
return <<'END';
200C	200D	Join_Control
END
