# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{InCjkUnifiedIdeographsExtensionB} (and fuzzy permutations)
# 
# Meaning: Block 'CJK Unified Ideographs Extension B'
#
return <<'END';
20000	2A6DF	CJK Unified Ideographs Extension B
END
